@@ -174,10 +174,14 @@ add_priors.bvarmodel <- function(object,
   
   # Checks - Coefficient priors ----
   if (!is.null(coef)) {
-    if (!is.null(coef$v_i)) {
-      if (coef$v_i < 0) {
+    if (!is.null(coef[["v_i"]])) {
+      if (coef[["v_i"]] < 0) {
         stop("Argument 'v_i' must be at least 0.")
-      }  
+      }
+      # Define "v_i_det" if not specified (needed for a check later)
+      if (is.null(coef[["v_i_det"]])) {
+        coef[["v_i_det"]] <- coef[["v_i"]]
+      }
     } else {
       if (!any(c("minnesota", "ssvs") %in% names(coef))) {
         stop("If 'coef$v_i' is not specified, at least 'coef$minnesota' or 'coef$ssvs' must be specified.")
@@ -185,9 +189,9 @@ add_priors.bvarmodel <- function(object,
     }
   }
   
-  if (!is.null(coef$const)) {
-    if (class(coef$const) == "character") {
-      if (!coef$const %in% c("first", "mean")) {
+  if (!is.null(coef[["const"]])) {
+    if ("character" %in% class(coef[["const"]])) {
+      if (!coef[["const"]] %in% c("first", "mean")) {
         stop("Invalid specificatin of coef$const.")
       }
     }
@@ -308,7 +312,7 @@ add_priors.bvarmodel <- function(object,
         use_bvs_error <- TRUE 
       }
     }
-    if (coef$v_i == 0 | (coef$v_i_det == 0 & !bvs$exclude_det)) {
+    if (coef[["v_i"]] == 0 | (coef[["v_i_det"]] == 0 & !bvs[["exclude_det"]])) {
       warning("Using BVS with an uninformative prior is not recommended.")
     }
   }
@@ -396,7 +400,7 @@ add_priors.bvarmodel <- function(object,
           pos <- which(dimnames(object[[i]][["data"]][["Z"]])[[2]] == "const")
           
           if (length(pos) == 1) {
-            if (class(coef[["const"]]) == "character") {
+            if ("character" %in% class(coef[["const"]])) {
               if (coef[["const"]] == "first") {
                 mu[, pos] <- object[[i]][["data"]][["Y"]][1, ]
               }
@@ -404,7 +408,7 @@ add_priors.bvarmodel <- function(object,
                 mu[, pos] <- colMeans(object[[i]][["data"]][["Y"]])
               }
             }
-            if (class(coef[["const"]]) == "numeric") {
+            if ("numeric" %in% class(coef[["const"]])) {
               mu[, pos] <- coef[["const"]]
             } 
           }
@@ -541,7 +545,7 @@ add_priors.bvarmodel <- function(object,
         object[[i]][["priors"]][["sigma"]]$sigma_i = minn[["sigma_i"]]
       }
       
-      if (class(help_df) == "character") {
+      if ("character" %in% class(help_df)) {
         if (grepl("k", help_df)) {
           # Transform character specification to expression and evaluate
           help_df <- eval(parse(text = help_df))
